/*
  Copyright 2008 CNRS
  Author: Florent Lamiraux
*/

#include "kppInterfaceDisplay/kppCommandCreateGenoMTraj.h"

#include "KineoModel/kppComponentParameter.h"
#include "KineoModel/kppValue.h"
#include "KineoModel/kppComponentClassFilter.h"

#include "posterLib.h"
#include "trajPoster.h"

// Select level of verbosity at configuration.
#if DEBUG==2
#define ODEBUG2(x) std::cout << "CkppCommandCreateGenoMTraj:" << x << std::endl
#define ODEBUG1(x) std::cerr << "CkppCommandCreateGenoMTraj:" << x << std::endl
#elif DEBUG==1
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "CkppCommandCreateGenoMTraj:" << x << std::endl
#else
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif

CkppCommandCreateGenoMTrajShPtr CkppCommandCreateGenoMTraj::create(CkppInterfaceDisplay *inKppInterface)
{
  CkppCommandCreateGenoMTraj* commandPtr = new CkppCommandCreateGenoMTraj(inKppInterface);
  CkppCommandCreateGenoMTrajShPtr commandShPtr(commandPtr);

  if (commandPtr->init(commandShPtr) == KD_ERROR) {
    commandShPtr.reset();
  }
  return commandShPtr;
}

CkppCommandCreateGenoMTrajShPtr CkppCommandCreateGenoMTraj::createCopy(const CkppCommandCreateGenoMTrajConstShPtr& inCommand)
{
  CkppCommandCreateGenoMTraj* commandPtr = new CkppCommandCreateGenoMTraj(*inCommand);
  CkppCommandCreateGenoMTrajShPtr commandShPtr(commandPtr);

  if (commandPtr->init(commandShPtr) == KD_ERROR) {
    commandShPtr.reset();
  }
  return commandShPtr;
}

CkppCommandCreateGenoMTraj::~CkppCommandCreateGenoMTraj()
{
}

bool CkppCommandCreateGenoMTraj::isUndoable() const
{
  return true;
}

CkppCommandShPtr CkppCommandCreateGenoMTraj::clone() const
{
  return CkppCommandCreateGenoMTraj::createCopy(attWeakPtr.lock());
}
  
unsigned int CkppCommandCreateGenoMTraj::countParameters() const
{
  return PARAMETER_COUNT;
}
  
CkppParameterConstShPtr CkppCommandCreateGenoMTraj::parameter(unsigned int inRank) const
{
  CkppParameterShPtr result;
  CkppComponentShPtr nullComponent;

  switch(inRank) {   
  case ROBOT:
    result = CkppComponentParameter::create("a robot", CkppComponentClassFilter<CkppDeviceComponent>());
    break;
  default:
    KIT_ASSERT( false );
  }
  return result;
}

CkppCommandCreateGenoMTraj::CkppCommandCreateGenoMTraj(CkppInterfaceDisplay *inKppInterface) :
  attKppInterface(inKppInterface)
{
}

CkppCommandCreateGenoMTraj::CkppCommandCreateGenoMTraj(const CkppCommandCreateGenoMTraj& inCommand) :
  CkppCommand(inCommand), attKppInterface(inCommand.attKppInterface)
{
}

ktStatus CkppCommandCreateGenoMTraj::init(const CkppCommandCreateGenoMTrajWkPtr& inWeakPtr)
{
  if (CkppCommand::init(inWeakPtr) == KD_ERROR) {
    return KD_ERROR;
  }
  attWeakPtr = inWeakPtr;
  return KD_OK;
}

ktStatus CkppCommandCreateGenoMTraj::doExecute()
{
  // Get robot
  CkppComponentShPtr component(paramValue(parameter(ROBOT)).componentValue());
  CkppDeviceComponentShPtr deviceComponent = KIT_DYNAMIC_PTR_CAST(CkppDeviceComponent, component);

  // Find problem id corresponding to robot.
  ChppPlanner* hppPlanner = attKppInterface->hppPlanner();
  if (!hppPlanner) {
    ODEBUG1(" no ChppPlanner object.");
    return KD_ERROR;
  }

  bool problemFound = false;
  unsigned int iProblem;
  for (iProblem=0; iProblem < hppPlanner->getNbHppProblems(); iProblem++) {
    if (hppPlanner->robotIthProblem(iProblem) == deviceComponent) {
      problemFound = true;
      break;
    }
  }
  if (!problemFound) {
    ODEBUG1(" This robot does not belong to ChppPlanner object.");
    return KD_ERROR;
  }
  CkwsPathShPtr path = CkwsPath::create(deviceComponent);

  std::string trajPosterName;

  std::cout << "Type name of poster: ";
  std::cin >> trajPosterName;

  POSTER_ID posterId;
  if (posterFind (const_cast<char*>(trajPosterName.c_str()), &posterId) != OK) {
    ODEBUG1(" poster " << std::string(trajPosterName) << " not found.");
    return KD_ERROR;
  }

  // Register poster in mapping
  
  ROBOT_TRAJ_POSTER_STR robotTraj;
  if (posterRead(posterId, 0, &robotTraj, sizeof(ROBOT_TRAJ_POSTER_STR)) !=
      sizeof(ROBOT_TRAJ_POSTER_STR)) {
    ODEBUG1(" failed to read " << std::string(trajPosterName) << ".");
    return KD_ERROR;
  }
  if (hppPlanner->addPath(iProblem, path) != KD_OK) {
    ODEBUG1(": failed to add path to ChppPlanner object.");
    return KD_ERROR;
  }
  attKppInterface->addPosterNameToMapping(path, std::string(trajPosterName));

  if (attKppInterface->updatePathFromPoster(path) != KD_OK) {
    return KD_ERROR;
  }
  return KD_OK;
}

